Option Strict Off
Option Explicit On

Public Class N5980A
   
    'm_Client holds the TcpClient that is used to establish 
    'the remote connection.
    Private m_Client As TcpClient
    'm_Stream holds the NetworkStream that provides 
    'the communication link.
    Private m_stream As NetworkStream


    Private Sub ConnectButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ConnectButton.Click
        'Connect to the instrument with a given hostname and 
        'port number. Both values can be retrieved by going to
        'the menu bar in the N5980A user interface:
        'Connection-->Info
        'Use the IP Address (or hostname of the PC that runs
        'the N5980A User Interface) and use the port number
        'of the User Interface software instance to
        'communicate with.
        Dim port_number As Integer
        port_number = 5025
        Try
            'Resolve address of the given hostname
            Dim ipHostEntry As IPHostEntry
            Dim ipAddress As IPAddress

            ipHostEntry = Dns.GetHostEntry(host_address.Text)
            ipAddress = ipHostEntry.AddressList(0)

            'create TcpClient and connect it to the instrument
            m_Client = New TcpClient()
            m_Client.Connect(New IPEndPoint(ipAddress, port_number))
            m_stream = m_Client.GetStream()
            Connected.Text = "Instrument is connected"
            ConnectButton.Enabled = False
            DisconnectButton.Enabled = True
            GroupBox2.Enabled = True
            GroupBox3.Enabled = True
            Exit Sub

        Catch err As System.Exception
            MessageBox.Show("Open failed on " + host_address.Text + " " + err.Message, "MyN5980A", MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try
    End Sub


    Private Sub DisconnectButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles DisconnectButton.Click
        'Close the connection with the N5980A software.
        m_Client.Close()
        'From here on, it is no longer possible to communicate with the
        'instrument.
        Connected.Text = "Instrument is disconnected"
        ConnectButton.Enabled = True
        DisconnectButton.Enabled = False
        GroupBox2.Enabled = False
        GroupBox3.Enabled = False
    End Sub


    Private Function SendCommand(ByVal scpi_string As String) As String
        'Send a N5980A SCPI command that is contained in
        'String to the Windows socket and 
        'return the response (if it was a query),
        'or an empty string if there is no response.

        Dim i As Integer
        Dim numRead As Integer
        Dim sb As New StringBuilder
        Dim bufferin() As Byte = New Byte(64 * 1024) {}
        Dim bufferout() As Byte = New Byte(scpi_string.Length) {}

        For i = 0 To (scpi_string.Length - 1)
            bufferout(i) = Convert.ToByte(scpi_string(i))
        Next
        bufferout(scpi_string.Length) = Convert.ToByte(Chr(10))
        m_stream.Write(bufferout, 0, bufferout.Length)

        'If the string containes a '?', then it is a query and
        'the response needs to be read from the instrument
        SendCommand = ""
        If scpi_string.IndexOf("?"c) <> -1 Then
            'Read resonse from network stream
            numRead = m_stream.Read(bufferin, 0, bufferin.Length)
            'if the response contained any bytes, then convert them
            'to a valid string
            If (numRead > 0) Then
                For i = 0 To numRead
                    sb.Append(Convert.ToChar(bufferin(i)))
                    'response = response + Convert.ToChar(bufferin(i))
                Next
                SendCommand = sb.ToString
            End If
        End If
    End Function


    Private Sub N5980A_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        'Initialize some of the buttons
        ConnectButton.Enabled = True
        DisconnectButton.Enabled = False
        host_address.SelectedIndex = 1 'select which address is shown as default
        Connected.Text = "Instrument is disconnected"
        frequency.SelectedIndex = 1
        GroupBox2.Enabled = False
        GroupBox3.Enabled = False
    End Sub

    
    Private Sub configureButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles m_configureButton.Click
        Dim command_string As String

        Select Case frequency.Text.ToUpper()
            Case "FAST ETHERNET"
                command_string = ":FREQ FE"
            Case "OC-3"
                command_string = ":FREQ OC3"
            Case "OC-12"
                command_string = ":FREQ OC12"
            Case "OC-48"
                command_string = ":FREQ OC48"
            Case "OC-48 WITH FEC"
                command_string = ":FREQ OC48FEC"
            Case "FCX1"
                command_string = ":FREQ FC1"
            Case "FCX2"
                command_string = ":FREQ FC2"
            Case "GIGABIT ETHERNET"
                command_string = ":FREQ GBE1"
            Case "XAUI"
                command_string = ":FREQ XAUI"
            Case Else
                command_string = ":FREQ FE"
        End Select
        SendCommand(command_string)
    End Sub


    Private Sub ReadSettingsButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ReadSettingsButton.Click
        Frequency2.Text = SendCommand(":FREQ?")
        PGPatternType.Text = SendCommand("SOUR1:PATT?")
        EDPatternType.Text = SendCommand("SENS1:PATT?")
        PGVoltageLevel.Text = SendCommand("SOUR1:VOLT?")
    End Sub
End Class
